package de.szut.casino.coinflip;

import de.szut.casino.exceptionHandling.exceptions.InsufficientFundsException;
import de.szut.casino.exceptionHandling.exceptions.UserNotFoundException;
import de.szut.casino.shared.service.BalanceService;
import de.szut.casino.user.UserEntity;
import de.szut.casino.user.UserService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.springframework.http.ResponseEntity;

import java.math.BigDecimal;
import java.util.Optional;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

class CoinflipControllerTest {
    @Mock
    private UserService userService;
    @Mock
    private BalanceService balanceService;
    @Mock
    private CoinflipService coinflipService;
    @InjectMocks
    private CoinflipController coinflipController;

    private UserEntity user;
    private CoinflipDto coinflipDto;

    @BeforeEach
    void setUp() {
        MockitoAnnotations.openMocks(this);
        user = new UserEntity();
        user.setBalance(BigDecimal.valueOf(100));
        coinflipDto = new CoinflipDto(BigDecimal.valueOf(10), CoinSide.HEAD);
    }

    @Test
    void testCoinFlip_success() {
        when(userService.getCurrentUser()).thenReturn(Optional.of(user));
        when(balanceService.hasFunds(user, coinflipDto)).thenReturn(true);
        CoinflipResult result = new CoinflipResult(true, BigDecimal.valueOf(20), CoinSide.HEAD);
        when(coinflipService.play(user, coinflipDto)).thenReturn(result);

        ResponseEntity<Object> response = coinflipController.coinFlip(coinflipDto);

        assertEquals(200, response.getStatusCodeValue());
        assertEquals(result, response.getBody());
    }

    @Test
    void testCoinFlip_userNotFound() {
        when(userService.getCurrentUser()).thenReturn(Optional.empty());
        assertThrows(UserNotFoundException.class, () -> coinflipController.coinFlip(coinflipDto));
    }

    @Test
    void testCoinFlip_insufficientFunds() {
        when(userService.getCurrentUser()).thenReturn(Optional.of(user));
        when(balanceService.hasFunds(user, coinflipDto)).thenReturn(false);
        assertThrows(InsufficientFundsException.class, () -> coinflipController.coinFlip(coinflipDto));
    }
} 